/* -*- c-basic-offset: 3; mode: c++ -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 * 
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

/**
 * @author Alexandre Duret-Lutz - 1999-10-22
 * @author Rgis Clouard - 2001-06-08
 */

#include <pandore.h>
using namespace pandore;

/**
 * @file phistogram.cpp
 * Computes the histogram.
 * (l'histogramme des images signes est decale de -min)
 */

Errc PHistogram( const Img1duc &ims, Img1dsl &hg ) {
   Img1duc::ValueType *ps=ims.Vector();
   const Img1duc::ValueType *pf=ims.Vector()+ims.VectorSize();
   
   for (int i= 0; i < hg.Width(); hg[++i]=0 ) ;
   
   for ( ; ps<pf ; ps++ ) {
      int pos = *ps - Limits<Img1duc::ValueType>::min();
      if (hg[pos] < Limits<Img1dsl::ValueType>::max() ) { // sature ?
	 ++hg[pos];
      }
   }
   
   return SUCCESS;
}

Errc PHistogram( const Img2duc &ims, Img1dsl &hg ) {
   Img2duc::ValueType *ps=ims.Vector();
   const Img2duc::ValueType *pf=ims.Vector()+ims.VectorSize();
   
   for (int i= 0; i < hg.Width(); hg[++i]=0 ) ;
   
   for ( ; ps<pf ; ps++ ) {
      int pos = *ps - Limits<Img2duc::ValueType>::min();
      if (hg[pos] < Limits<Img1dsl::ValueType>::max() ) { // sature ?
	 ++hg[pos];
      }
   }
   
   return SUCCESS;
}

Errc PHistogram( const Img3duc &ims, Img1dsl &hg ) {
   Img3duc::ValueType *ps=ims.Vector();
   const Img3duc::ValueType *pf=ims.Vector()+ims.VectorSize();
   
   for (int i= 0; i < hg.Width(); hg[++i]=0 ) ;
   
   for ( ; ps<pf ; ps++ ) {
      int pos = *ps - Limits<Img3duc::ValueType>::min();
      if (hg[pos] < Limits<Img1dsl::ValueType>::max() ) { // sature ?
	 ++hg[pos];
      }
   }
   
   return SUCCESS;
}


#ifdef MAIN
#define USAGE	"usage: %s [-m mask] [im_in|-] [im_out|-]"
#define PARC	0
#define FINC	1
#define FOUTC	1
#define MASK	1

int main( int argc, char *argv[] ) {
   Errc result;                // The result code of the execution.
   Pobject* mask;              // The region map.
   Pobject* objin[FINC + 1];   // The input objects.
   Pobject* objs[FINC + 1];    // The source objects masked.
   Pobject* objout[FOUTC + 1]; // The output object.
   Pobject* objd[FOUTC + 1];   // The result object of the execution.
   char* parv[PARC + 1];       // The input parameters.

   ReadArgs(argc, argv, PARC, FINC, FOUTC, &mask, objin, objs, objout, objd, parv, USAGE, MASK); 
   if ( objs[0]->Type() == Po_Img1duc ) {
      Img1duc* const ims1=(Img1duc*)objs[0];
      objd[0]=new Img1dsl(Limits<Img1duc::ValueType>::max() - Limits<Img1duc::ValueType>::min() + 1);
      Img1dsl* const hg=(Img1dsl*)objd[0];
      result=PHistogram(*ims1,*hg);
      goto end;
   }
   if ( objs[0]->Type() == Po_Img2duc ) {
      Img2duc* const ims1=(Img2duc*)objs[0];
      objd[0]=new Img1dsl(Limits<Img2duc::ValueType>::max() - Limits<Img2duc::ValueType>::min() + 1);
      Img1dsl* const hg=(Img1dsl*)objd[0];
      result=PHistogram(*ims1,*hg);
      goto end;
   }
   if ( objs[0]->Type() == Po_Img3duc ) {
      Img3duc* const ims1=(Img3duc*)objs[0];
      objd[0]=new Img1dsl(Limits<Img3duc::ValueType>::max() - Limits<Img3duc::ValueType>::min() + 1);
      Img1dsl* const hg=(Img1dsl*)objd[0];
      result=PHistogram(*ims1,*hg);
      goto end;
   }
  {
     PrintErrorFormat(objin, FINC, argv); 
     result = FAILURE; 
  }	

end:
  if (result) {
	WriteArgs(argc, argv, PARC, FINC, FOUTC, &mask, objin, objs, objout, objd, MASK); 
  }
  Exit(result); 
  return 0; 
}
#endif
