/* -*- c-basic-offset: 3; mode: c++ -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 * 
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

/**
 * @author Franois Angot - 1998-07-01
 * @author Regis Clouard - 2004-07-30
 * @author Francois Lecellier - 2010-04-22 (fix bug on parameter)
 */

#include <pandore.h>
using namespace pandore;

/**
 * @file pflip.cpp
 * Construit le symetrique d'une image.
 */










Errc PFlip( const Img2duc &ims, Img2duc &imd, int direction ) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[ims.Height()-1-p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip( const Img2dsl &ims, Img2dsl &imd, int direction ) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[ims.Height()-1-p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip( const Img2dsf &ims, Img2dsf &imd, int direction ) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[ims.Height()-1-p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip( const Img3duc &ims, Img3duc &imd, int direction ) {
   Point3d p;
  
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.z][p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.z][ims.Height()-1-p.y][p.x];
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[ims.Depth()-1-p.z][p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
  
   return SUCCESS;
}

Errc PFlip( const Img3dsl &ims, Img3dsl &imd, int direction ) {
   Point3d p;
  
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.z][p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.z][ims.Height()-1-p.y][p.x];
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[ims.Depth()-1-p.z][p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
  
   return SUCCESS;
}

Errc PFlip( const Img3dsf &ims, Img3dsf &imd, int direction ) {
   Point3d p;
  
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.z][p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[p.z][ims.Height()-1-p.y][p.x];
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd[p] = ims[ims.Depth()-1-p.z][p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
  
   return SUCCESS;
}


Errc PFlip( const Imc2duc &ims, Imc2duc &imd, int direction ) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.y][ims.Width()-1-p.x];
	 imd.Y[p] = ims.Y[p.y][ims.Width()-1-p.x];
	 imd.Z[p] = ims.Z[p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[ims.Height()-1-p.y][p.x];
	 imd.Y[p] = ims.Y[ims.Height()-1-p.y][p.x];
	 imd.Z[p] = ims.Z[ims.Height()-1-p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip( const Imc2dsl &ims, Imc2dsl &imd, int direction ) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.y][ims.Width()-1-p.x];
	 imd.Y[p] = ims.Y[p.y][ims.Width()-1-p.x];
	 imd.Z[p] = ims.Z[p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[ims.Height()-1-p.y][p.x];
	 imd.Y[p] = ims.Y[ims.Height()-1-p.y][p.x];
	 imd.Z[p] = ims.Z[ims.Height()-1-p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip( const Imc2dsf &ims, Imc2dsf &imd, int direction ) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.y][ims.Width()-1-p.x];
	 imd.Y[p] = ims.Y[p.y][ims.Width()-1-p.x];
	 imd.Z[p] = ims.Z[p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[ims.Height()-1-p.y][p.x];
	 imd.Y[p] = ims.Y[ims.Height()-1-p.y][p.x];
	 imd.Z[p] = ims.Z[ims.Height()-1-p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip( const Imc3duc &ims, Imc3duc &imd, int direction ) {
   Point3d p;
  
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.z][p.y][ims.Width()-1-p.x];
	 imd.Y[p] = ims.Y[p.z][p.y][ims.Width()-1-p.x];
	 imd.Z[p] = ims.Z[p.z][p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.z][ims.Height()-1-p.y][p.x];
	 imd.Y[p] = ims.Y[p.z][ims.Height()-1-p.y][p.x];
	 imd.Z[p] = ims.Z[p.z][ims.Height()-1-p.y][p.x];
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[ims.Depth()-1-p.z][p.y][p.x];
	 imd.Y[p] = ims.Y[ims.Depth()-1-p.z][p.y][p.x];
	 imd.Z[p] = ims.Z[ims.Depth()-1-p.z][p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   
   return SUCCESS;
}

Errc PFlip( const Imc3dsl &ims, Imc3dsl &imd, int direction ) {
   Point3d p;
  
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.z][p.y][ims.Width()-1-p.x];
	 imd.Y[p] = ims.Y[p.z][p.y][ims.Width()-1-p.x];
	 imd.Z[p] = ims.Z[p.z][p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.z][ims.Height()-1-p.y][p.x];
	 imd.Y[p] = ims.Y[p.z][ims.Height()-1-p.y][p.x];
	 imd.Z[p] = ims.Z[p.z][ims.Height()-1-p.y][p.x];
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[ims.Depth()-1-p.z][p.y][p.x];
	 imd.Y[p] = ims.Y[ims.Depth()-1-p.z][p.y][p.x];
	 imd.Z[p] = ims.Z[ims.Depth()-1-p.z][p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   
   return SUCCESS;
}

Errc PFlip( const Imc3dsf &ims, Imc3dsf &imd, int direction ) {
   Point3d p;
  
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.z][p.y][ims.Width()-1-p.x];
	 imd.Y[p] = ims.Y[p.z][p.y][ims.Width()-1-p.x];
	 imd.Z[p] = ims.Z[p.z][p.y][ims.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[p.z][ims.Height()-1-p.y][p.x];
	 imd.Y[p] = ims.Y[p.z][ims.Height()-1-p.y][p.x];
	 imd.Z[p] = ims.Z[p.z][ims.Height()-1-p.y][p.x];
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 imd.X[p] = ims.X[ims.Depth()-1-p.z][p.y][p.x];
	 imd.Y[p] = ims.Y[ims.Depth()-1-p.z][p.y][p.x];
	 imd.Z[p] = ims.Z[ims.Depth()-1-p.z][p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   
   return SUCCESS;
}


Errc PFlip(Imx2duc &ims, Imx2duc &imd, int direction) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][ims.Height()-1-p.y][p.x];
	 }
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip(Imx2dsl &ims, Imx2dsl &imd, int direction) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][ims.Height()-1-p.y][p.x];
	 }
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip(Imx2dsf &ims, Imx2dsf &imd, int direction) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 1 :
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][ims.Height()-1-p.y][p.x];
	 }
      }
      break;
   default: return FAILURE;
   }
   return SUCCESS;
}

Errc PFlip( const Imx3duc &ims, Imx3duc &imd, int direction ) {
   Point3d p;
   
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   default: return FAILURE;
   }
   
   return SUCCESS;
}


Errc PFlip( const Imx3dsl &ims, Imx3dsl &imd, int direction ) {
   Point3d p;
   
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   default: return FAILURE;
   }
   
   return SUCCESS;
}


Errc PFlip( const Imx3dsf &ims, Imx3dsf &imd, int direction ) {
   Point3d p;
   
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 1 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   case 2 :
   for (p.z = 0; p.z < ims.Depth(); p.z++)
   for (p.y = 0; p.y < ims.Height(); p.y++)
   for (p.x = 0; p.x < ims.Width(); p.x++)
      {
	 for (int b = 0; b < ims.Bands(); b++) {
	    imd[b][p] = ims[b][p.z][p.y][ims.Width()-1-p.x];
	 }
      }
      break;
   default: return FAILURE;
   }
   
   return SUCCESS;
}



Errc PFlip( const Reg2d &rgs, Reg2d &rgd, int direction ) {
   Point2d p;
   
   switch(direction) {
   case 0 :
   for (p.y = 0; p.y < rgs.Height(); p.y++)
   for (p.x = 0; p.x < rgs.Width(); p.x++)
      {
	 rgd[p] = rgs[p.y][rgs.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.y = 0; p.y < rgs.Height(); p.y++)
   for (p.x = 0; p.x < rgs.Width(); p.x++)
      {
	 rgd[p] = rgs[rgs.Height()-1-p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   rgd.Labels(rgs.Labels());
   return SUCCESS;
}

Errc PFlip( const Reg3d &rgs, Reg3d &rgd, int direction ) {
   Point3d p;
  
   switch(direction) {
   case 0 :
   for (p.z = 0; p.z < rgs.Depth(); p.z++)
   for (p.y = 0; p.y < rgs.Height(); p.y++)
   for (p.x = 0; p.x < rgs.Width(); p.x++)
      {
	 rgd[p] = rgs[p.z][p.y][rgs.Width()-1-p.x];
      }
      break;
   case 1 :
   for (p.z = 0; p.z < rgs.Depth(); p.z++)
   for (p.y = 0; p.y < rgs.Height(); p.y++)
   for (p.x = 0; p.x < rgs.Width(); p.x++)
      {
	 rgd[p] = rgs[p.z][rgs.Height()-1-p.y][p.x];
      }
      break;
   case 2 :
   for (p.z = 0; p.z < rgs.Depth(); p.z++)
   for (p.y = 0; p.y < rgs.Height(); p.y++)
   for (p.x = 0; p.x < rgs.Width(); p.x++)
      {
	 rgd[p] = rgs[rgs.Depth()-1-p.z][p.y][p.x];
      }
      break;
   default: return FAILURE;
   }
   rgd.Labels(rgs.Labels());  
   return SUCCESS;
}


#ifdef MAIN
#define	USAGE	"usage: %s direction [-m mask] [im_in|-] [im_out|-]"
#define	PARC	1
#define	FINC	1
#define	FOUTC	1
#define	MASK	1

int main( int argc, char *argv[] ) {
   Errc result;                // The result code of the execution.
   Pobject* mask;              // The region map.
   Pobject* objin[FINC + 1];   // The input objects.
   Pobject* objs[FINC + 1];    // The source objects masked.
   Pobject* objout[FOUTC + 1]; // The output object.
   Pobject* objd[FOUTC + 1];   // The result object of the execution.
   char* parv[PARC + 1];       // The input parameters.

   ReadArgs(argc, argv, PARC, FINC, FOUTC, &mask, objin, objs, objout, objd, parv, USAGE, MASK); 
   if (objs[0]->Type() == Po_Img2duc) {
      Img2duc* const ims = (Img2duc*)objs[0];
      objd[0] = new Img2duc(ims->Props());
      Img2duc* const imd = (Img2duc*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Img2dsl) {
      Img2dsl* const ims = (Img2dsl*)objs[0];
      objd[0] = new Img2dsl(ims->Props());
      Img2dsl* const imd = (Img2dsl*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Img2dsf) {
      Img2dsf* const ims = (Img2dsf*)objs[0];
      objd[0] = new Img2dsf(ims->Props());
      Img2dsf* const imd = (Img2dsf*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Img3duc) {
      Img3duc* const ims = (Img3duc*)objs[0];
      objd[0] = new Img3duc(ims->Props());
      Img3duc* const imd = (Img3duc*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Img3dsl) {
      Img3dsl* const ims = (Img3dsl*)objs[0];
      objd[0] = new Img3dsl(ims->Props());
      Img3dsl* const imd = (Img3dsl*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Img3dsf) {
      Img3dsf* const ims = (Img3dsf*)objs[0];
      objd[0] = new Img3dsf(ims->Props());
      Img3dsf* const imd = (Img3dsf*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imc2duc) {
      Imc2duc* const ims = (Imc2duc*)objs[0];
      objd[0] = new Imc2duc(ims->Props());
      Imc2duc* const imd = (Imc2duc*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imc2dsl) {
      Imc2dsl* const ims = (Imc2dsl*)objs[0];
      objd[0] = new Imc2dsl(ims->Props());
      Imc2dsl* const imd = (Imc2dsl*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imc2dsf) {
      Imc2dsf* const ims = (Imc2dsf*)objs[0];
      objd[0] = new Imc2dsf(ims->Props());
      Imc2dsf* const imd = (Imc2dsf*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imc3duc) {
      Imc3duc* const ims = (Imc3duc*)objs[0];
      objd[0] = new Imc3duc(ims->Props());
      Imc3duc* const imd = (Imc3duc*)objd[0];
      
      result  =PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imc3dsl) {
      Imc3dsl* const ims = (Imc3dsl*)objs[0];
      objd[0] = new Imc3dsl(ims->Props());
      Imc3dsl* const imd = (Imc3dsl*)objd[0];
      
      result  =PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imc3dsf) {
      Imc3dsf* const ims = (Imc3dsf*)objs[0];
      objd[0] = new Imc3dsf(ims->Props());
      Imc3dsf* const imd = (Imc3dsf*)objd[0];
      
      result  =PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imx2duc) {
      Imx2duc* const ims = (Imx2duc*)objs[0];
      objd[0] = new Imx2duc(ims->Props());
      Imx2duc* const imd = (Imx2duc*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imx2dsl) {
      Imx2dsl* const ims = (Imx2dsl*)objs[0];
      objd[0] = new Imx2dsl(ims->Props());
      Imx2dsl* const imd = (Imx2dsl*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imx2dsf) {
      Imx2dsf* const ims = (Imx2dsf*)objs[0];
      objd[0] = new Imx2dsf(ims->Props());
      Imx2dsf* const imd = (Imx2dsf*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imx3duc) {
      Imx3duc* const ims = (Imx3duc*)objs[0];
      objd[0] = new Imx3duc(ims->Props());
      Imx3duc* const imd = (Imx3duc*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imx3dsl) {
      Imx3dsl* const ims = (Imx3dsl*)objs[0];
      objd[0] = new Imx3dsl(ims->Props());
      Imx3dsl* const imd = (Imx3dsl*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Imx3dsf) {
      Imx3dsf* const ims = (Imx3dsf*)objs[0];
      objd[0] = new Imx3dsf(ims->Props());
      Imx3dsf* const imd = (Imx3dsf*)objd[0];
      
      result = PFlip(*ims, *imd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Reg2d) {
      Reg2d* const rgs = (Reg2d*)objs[0];
      objd[0] = new Reg2d(rgs->Props());
      Reg2d* const rgd = (Reg2d*)objd[0];
      
      result = PFlip(*rgs, *rgd, atoi(parv[0]));
      goto end;
   }
   if (objs[0]->Type() == Po_Reg3d) {
      Reg3d* const rgs = (Reg3d*)objs[0];
      objd[0] = new Reg3d(rgs->Props());
      Reg3d* const rgd = (Reg3d*)objd[0];
      
      result = PFlip(*rgs, *rgd, atoi(parv[0]));
      goto end;
   }
  {
     PrintErrorFormat(objin, FINC, argv); 
     result = FAILURE; 
  }	

end:
  if (result) {
	WriteArgs(argc, argv, PARC, FINC, FOUTC, &mask, objin, objs, objout, objd, MASK); 
  }
  Exit(result); 
  return 0; 
}
#endif
